<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://ozthegreat.io/wordpress/wpml-editor-languages
 * @since      1.0.0
 *
 * @package    Wpml_Editor_Languages
 * @subpackage Wpml_Editor_Languages/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wpml_Editor_Languages
 * @subpackage Wpml_Editor_Languages/includes
 * @author     Edward D <edward@ozthegreat.io>
 */
class Wpml_Editor_Languages {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wpml_Editor_Languages_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = WPML_EDITOR_LANGUAGES_PLUGIN_NAME;
		$this->version     = WPML_EDITOR_LANGUAGES_VERSION;

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wpml_Editor_Languages_Loader. Orchestrates the hooks of the plugin.
	 * - Wpml_Editor_Languages_i18n. Defines internationalization functionality.
	 * - Wpml_Editor_Languages_Admin. Defines all hooks for the admin area.
	 * - Wpml_Editor_Languages_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpml-editor-languages-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpml-editor-languages-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wpml-editor-languages-admin.php';

		$this->loader = new Wpml_Editor_Languages_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wpml_Editor_Languages_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wpml_Editor_Languages_i18n();
		$plugin_i18n->set_domain( 'wpml-editor-languages' );

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wpml_Editor_Languages_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_init', $plugin_admin, 'set_allowed_languages', 10 );
		$this->loader->add_action( 'login_redirect', $plugin_admin, 'login_allowd_languages_redirect', 10, 3 );

		$this->loader->add_action( 'user_new_form', $plugin_admin, 'add_user_languages_persmissions', 10 );
		$this->loader->add_action( 'edit_user_profile', $plugin_admin, 'add_user_languages_persmissions', 10 );

		$this->loader->add_action( 'user_register', $plugin_admin, 'save_user_languages_allowed', 10 );
		$this->loader->add_action( 'personal_options_update', $plugin_admin, 'save_user_languages_allowed', 10 );
		$this->loader->add_action( 'edit_user_profile_update', $plugin_admin, 'save_user_languages_allowed', 10 );

		$this->loader->add_filter( 'wpml_admin_language_switcher_active_languages', $plugin_admin, 'set_language_switcher_languages', 10, 1 );
		$this->loader->add_filter( 'wpml_admin_language_switcher_items', $plugin_admin, 'remove_all_items_option', 10, 1 );
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wpml_Editor_Languages_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
